@@ -7,19 +7,45 @@ from mpi4py import MPI
 import scipy.sparse as sp
 import time
 from scipy.sparse.linalg import lgmres
-import scikits.umfpack as um
+#import scikits.umfpack as um
 import sys
-#from shapely.geometry import LineString
 import scipy
-from cachetools import cached
+from cachetools import cached,LRUCache
 from cachetools.keys import hashkey
 
-
 comm = MPI.COMM_WORLD
 
-@cached(cache={}, key=lambda F, k: hashkey(k))
-def get_SU(F,k):
-     return splu(k*F + sp.eye(F.shape[0]))
+cache_assemble = LRUCache(maxsize=1000)
+cache_compute_grad_data = LRUCache(maxsize=10000)
+cache_get_SU = LRUCache(maxsize=10000)
+cache_get_decomposed_directions = LRUCache(maxsize=10000)
+
+
+def clear_fourier_cache():
+
+   if comm.rank == 0:
+    cache_assemble.clear()
+    cache_compute_grad_data.clear()
+    cache_get_SU.clear()
+    cache_get_decomposed_directions.clear()
+
+def fourier_info(data):
+          print('                        FOURIER                 ',flush=True)   
+          print(colored(' -----------------------------------------------------------','green'),flush=True)
+          print(colored('  Iterations:                              ','green') + str(int(data[2])),flush=True)
+          print(colored('  Relative error:                          ','green') + '%.1E' % (data[1]),flush=True)
+          print(colored('  Fourier Thermal Conductivity [W/m/K]:    ','green') + str(round(data[0],3)),flush=True)
+          print(colored(' -----------------------------------------------------------','green'),flush=True)
+          print(" ")
+
+
+
+@cached(cache=cache_get_SU, key=lambda bundle, k: hashkey(k))
+def get_SU(bundle,k):
+
+     A = k*bundle[0] + sp.diags(bundle[1],format='csc')
+    
+     return splu(A)
 
 def get_key(ll,kappa):
 
@@ -31,45 +57,40 @@ def unpack(data):
     return data[0],np.array(data[1])
 
 
-def fix(F,B):
-
-   n_elems = F.shape[0]
-   scale = 1/F.max(axis=0).toarray()[0]
-   n = np.random.randint(n_elems)
-   scale[n] = 0
-   F.data = F.data * scale[F.indices]
-   F[n,n] = 1
-   B[n] = 0   
-
-   return scale
+def solve_fourier_single(argv):
+   """ Solve Fourier with a single set of kappa """
 
 
-def solve_fourier_single(kappa,**argv):
+   data = None
 
-   mesh = argv['mesh']
-   dim = int(mesh['meta'][2])
-   n_elems = int(mesh['meta'][0])
+   if comm.rank == 0:
 
-   kappa = -1 #it means that it will take the map from mesh
+    mesh = argv['geometry']
 
-   F,B = assemble(mesh,kappa)
+    mesh['elem_kappa_map'] = get_kappa_map_from_mat(**argv)
 
-   if argv.setdefault('fix',True): 
-       scale = fix(F,B)
-   else:
-       scale = []
+    kappa = -1 #it means that it will take the map from mesh
 
-   meta,temp,grad =  solve_convergence(argv,kappa,B,splu(F),scale=scale)
+    F,B,scale = assemble(mesh,kappa)
 
-   kappa_map = get_kappa_map(mesh,kappa)
+    meta,temp,grad =  solve_convergence(argv,kappa,B,splu(F),scale=scale)
 
-   flux = -np.einsum('cij,cj->ci',kappa_map,grad)
-   return {'flux_fourier':flux,'temperature_fourier':temp,'meta':np.array(meta),'grad':grad}
+    kappa_map = get_kappa_map(mesh,kappa)
+
+    flux = -np.einsum('cij,cj->ci',kappa_map,grad)
+
+    if argv['verbose']:
+     fourier_info(meta)
+   
+    data = {'meta':meta,'flux':flux,'temperature':temp,'grad':grad}
+
+   return data
 
 
 
 def get_kappa(mesh,i,j,ll,kappa):
 
+
    if i == j:
     return np.array(kappa[i])
    
@@ -83,7 +104,18 @@ def get_kappa(mesh,i,j,ll,kappa):
    kj = np.dot(normal,np.dot(kappa_j,normal))
    w  = mesh['interp_weigths'][ll]
 
-   kappa_loc = kj*kappa_i/(ki*(1-w) + kj*w)
+   if ll in mesh['interface_sides']:
+     dist   = mesh['dists'][ll,0:dim]*1e-9
+
+     h = 0.882e9
+
+     #B = ki*kj/h/np.linalg.norm(dist)
+     B = 0
+
+   else:  
+     B = 0
+
+   kappa_loc = kj*kappa_i/(ki*(1-w) + kj*w + B)
 
    return kappa_loc
 
@@ -102,9 +134,8 @@ def get_kappa_tensor(mesh,ll,kappa):
     return rot   
 
 
-@cached(cache={}, key=lambda mesh,kappa_ll:hashkey(kappa_ll))
+@cached(cache=cache_get_decomposed_directions, key=lambda mesh,kappa_ll:hashkey(kappa_ll))
 def get_decomposed_directions(mesh,kappa_ll):
-
      
     ll,kappa = unpack(kappa_ll)
 
@@ -133,8 +164,8 @@ def compute_laplacian(temp,**argv):
 
 
 
-@cached(cache={},key=lambda mesh,dummy:dummy)
-def compute_grad_data(mesh,dummy):
+@cached(cache=cache_compute_grad_data,key=lambda mesh,dummy:dummy)
+def compute_grad_data(mesh,TB):
 
      #Compute deltas-------------------   
      rr = []
@@ -142,29 +173,41 @@ def compute_grad_data(mesh,dummy):
        rr.append(i*[0])
 
      #this is wrong
-
      for ll in mesh['active_sides'] :
       kc1,kc2 = mesh['side_elem_map_vec'][ll]
-      ind1 = list(mesh['elem_side_map_vec'][kc1]).index(ll)
-      ind2 = list(mesh['elem_side_map_vec'][kc2]).index(ll)
+      ind1    = list(mesh['elem_side_map_vec'][kc1]).index(ll)
+      ind2    = list(mesh['elem_side_map_vec'][kc2]).index(ll)
+
+      delta = 0
       if ll in mesh['periodic_sides']:
          delta = mesh['periodic_side_values'][list(mesh['periodic_sides']).index(ll)]
       else: delta = 0  
-      rr[kc1][ind1] = [kc2,kc1,delta] 
-      rr[kc2][ind2] = [kc1,kc2,-delta] 
+
+      rr[kc1][ind1] = [kc2,kc1, delta] 
+      rr[kc2][ind2] = [kc1,kc2,-delta]
  
      return rr
 
 
 def compute_grad(temp,**argv):
 
-   mesh = argv['mesh'] 
-   
+   mesh = argv['geometry'] 
+
    rr = compute_grad_data(mesh,1)
 
    diff_temp = [[temp[j[0]]-temp[j[1]]+j[2] for j in f] for f in rr]
    
+
+   #Add boundary
+   #if 'TB' in argv.keys():
+   # TB  = argv['TB']
+   # for n,(eb,sb) in enumerate(zip(*(mesh['eb'],mesh['sb']))):
+   #   ind               = list(mesh['elem_side_map_vec'][eb]).index(sb)
+   #   diff_temp[eb][ind] = temp[eb]-TB[n]
+
    gradT = np.array([np.einsum('js,s->j',mesh['weigths'][k,:,:mesh['n_non_boundary_side_per_elem'][k]],np.array(dt)) for k,dt in enumerate(diff_temp)])
+
+
    return gradT
 
 
@@ -177,9 +220,9 @@ def compute_secondary_flux(temp,kappa_map,**argv):
    gradT = compute_grad(temp,**argv)
    #--------------
 
-   mesh = argv['mesh'] 
+   mesh = argv['geometry'] 
    dim = int(mesh['meta'][2])
-   n_elems = len(argv['mesh']['elems'])
+   n_elems = len(mesh['elems'])
 
    #-----------SAVE some time--------------------------------------
    v_non_orth = {}
@@ -214,9 +257,7 @@ def compute_secondary_flux(temp,kappa_map,**argv):
 
 def compute_diffusive_thermal_conductivity(temp,kappa_map,**argv):
     
-
-
-   mesh = argv['mesh']  
+   mesh = argv['geometry']  
 
    dim = int(mesh['meta'][2])
    kappa_eff = 0
@@ -232,6 +273,7 @@ def compute_diffusive_thermal_conductivity(temp,kappa_map,**argv):
     (v_orth,v_non_orth) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
 
     deltaT = temp[i] - temp[j] - 1
+
     kappa_eff -= v_orth *  deltaT * area
 
     if 'grad' in argv.keys():
@@ -246,9 +288,12 @@ def compute_diffusive_thermal_conductivity(temp,kappa_map,**argv):
 
 def solve_convergence(argv,kappa,B,SU,log=False,scale=[]):
 
+    argv.setdefault('max_fourier_error',1e-6) 
+    argv.setdefault('max_fourier_iter',10) 
+
     C  = np.zeros_like(B)
 
-    mesh = argv['mesh']  
+    mesh = argv['geometry']  
 
     kappa_map = get_kappa_map(mesh,kappa)
 
@@ -256,6 +301,7 @@ def solve_convergence(argv,kappa,B,SU,log=False,scale=[]):
     dim = int(mesh['meta'][2])
     grad = np.zeros((n_elems,dim))
 
+    
     n_iter = 0;kappa_old = 0;error = 1  
     while error > argv['max_fourier_error'] and \
                   n_iter < argv['max_fourier_iter'] :
@@ -264,13 +310,11 @@ def solve_convergence(argv,kappa,B,SU,log=False,scale=[]):
     
         if len(scale) > 0: RHS *=scale
 
-
         temp = SU.solve(RHS)
         temp = temp - (max(temp)+min(temp))/2.0
-        #print(min(temp),max(temp))
 
         kappa_eff = compute_diffusive_thermal_conductivity(temp,kappa_map,**argv)
-        #quit()
+
         error = abs((kappa_eff - kappa_old)/kappa_eff)
         kappa_old = kappa_eff
         n_iter +=1
@@ -281,6 +325,7 @@ def solve_convergence(argv,kappa,B,SU,log=False,scale=[]):
 
 def get_kappa_map(mesh,kappa):
 
+
     n_elems = len(mesh['elems'])
     dim = int(mesh['meta'][2])
 
@@ -298,13 +343,13 @@ def get_kappa_map(mesh,kappa):
         if kappa.ndim == 2: 
           kappa = np.tile(kappa,(n_elems,1,1))
 
+    kappa = kappa[:,:dim,:dim]
     return kappa
 
 
-@cached(cache={}, key=lambda mesh, kappa: hashkey(kappa))
+@cached(cache=cache_assemble, key=lambda mesh, kappa: hashkey(kappa))
 def assemble(mesh,kappa):
 
-
     kappa_map = get_kappa_map(mesh,kappa)
 
     n_elems = len(mesh['elems'])
@@ -322,6 +367,7 @@ def assemble(mesh,kappa):
       if not i == j:
      
        kappa_loc = get_kappa(mesh,i,j,ll,kappa_map)
+      
        (v_orth,dummy) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
 
        iff.append(i)
@@ -343,34 +389,36 @@ def assemble(mesh,kappa):
 
     F = sp.csc_matrix((np.array(dff),(np.array(iff),np.array(jff))),shape = (n_elems,n_elems))
 
-    return F,B
-
+    #This scale the matrix and fixed zero temperature to a random point
+    #scale = fix_instability(F,B)
+    scale = np.ones(n_elems)
+    return F,B,scale
 
 
 
 def solve_fourier(kappa,DeltaT,argv):
 
+    mesh = argv['geometry']
     argv['DeltaT'] = DeltaT
+    dim = int(mesh['meta'][2])
 
-    n_elems = len(argv['mesh']['elems'])
+    n_serial = len(kappa)
+    n_elems = len(mesh['elems'])
 
-    tf = shared_array(np.zeros((argv['n_serial'],argv['n_elems'])) if comm.rank == 0 else None)
-    tfg = shared_array(np.zeros((argv['n_serial'],argv['n_elems'],argv['dim'])) if comm.rank == 0 else None)
+    tf = shared_array(np.zeros((n_serial,n_elems)) if comm.rank == 0 else None)
+    tfg = shared_array(np.zeros((n_serial,n_elems,dim)) if comm.rank == 0 else None)
 
     if comm.rank == 0:
-     dim = int(argv['mesh']['meta'][2])
 
-
-     F,B = assemble(argv['mesh'],tuple(map(tuple,np.eye(dim))))
+     F,B,scale = assemble(mesh,tuple(map(tuple,np.eye(dim))))
 
      ratio_old = 0
      ratio = np.zeros_like(kappa)
      for m,k in enumerate(kappa):
          
-         BB = k*B+argv['DeltaT']
-         meta,tf[m,:],tfg[m,:,:] = solve_convergence(argv\
-                                                        ,k*np.eye(dim),BB,\
-                                                         get_SU(F,k))
+         BB = k*B+DeltaT
+         meta,tf[m,:],tfg[m,:,:] = solve_convergence(argv,k*np.eye(dim),BB,get_SU([F,scale],k),scale=scale)
+         #meta,tf[m,:],tfg[m,:,:] = solve_convergence(argv,k*np.eye(dim),BB,splu(k*F),scale=scale)
          kappaf = meta[0]
          ratio[m] = kappaf/k
 
@@ -383,16 +431,16 @@ def solve_fourier(kappa,DeltaT,argv):
 
      #Regularize---
      if argv.setdefault('experimental_multiscale',False):
-      kappa_map = get_kappa_map(argv['mesh'],np.eye(dim))
+      kappa_map = get_kappa_map(mesh,np.eye(dim))
       ratio_0 = compute_diffusive_thermal_conductivity(DeltaT,kappa_map,**argv)
       grad_DeltaT = compute_grad(DeltaT,**argv)
     
       #Regularize gradient
       for m1 in range(len(kappa))[::-1]:
         error = (ratio[m1] - ratio_0)/ratio[m1]
+        
         if (ratio[m1] - ratio_0)/ratio[m1] < 1e-1 and abs(ratio[m1]-ratio[m1-1])/ratio[m1] < 1e-1:
             tfg[:m1+1,:,:] = grad_DeltaT
-            tf[:m1+1,:] = DeltaT
             break
 
       #Regularize average temperature
@@ -403,6 +451,7 @@ def solve_fourier(kappa,DeltaT,argv):
             break
      #-----------------
 
+
     comm.Barrier()
     return tf,tfg
 
